@@ -1,6 +1,3 @@
-# -*- coding: utf-8 -*-
-from __future__ import division, print_function
-
 import sys
 
 from plumbum import colors
@@ -9,7 +6,7 @@ from .. import cli
 from .termsize import get_terminal_size
 
 
-class Image(object):
+class Image:
 
     __slots__ = "size char_ratio".split()
 
@@ -39,18 +36,19 @@ class Image(object):
 
         import PIL.Image
 
-        if double:
-            return self.show_pil_double(PIL.Image.open(filename))
-        else:
-            return self.show_pil(PIL.Image.open(filename))
+        return (
+            self.show_pil_double(PIL.Image.open(filename))
+            if double
+            else self.show_pil(PIL.Image.open(filename))
+        )
 
     def _init_size(self, im):
         """Return the expected image size"""
         if self.size is None:
             term_size = get_terminal_size()
             return self.best_aspect(im.size, term_size)
-        else:
-            return self.size
+
+        return self.size
 
     def show_pil(self, im):
         "Standard show routine"
@@ -60,9 +58,10 @@ class Image(object):
         for y in range(size[1]):
             for x in range(size[0] - 1):
                 pix = new_im.getpixel((x, y))
-                print(colors.bg.rgb(*pix), " ", sep="", end="")  # u'\u2588'
-            print(colors.reset, " ", sep="")
-        print(colors.reset)
+                sys.stdout.write(colors.bg.rgb(*pix) + " ")  # '\u2588'
+            sys.stdout.write(colors.reset + " \n")
+        sys.stdout.write(colors.reset + "\n")
+        sys.stdout.flush()
 
     def show_pil_double(self, im):
         "Show double resolution on some fonts"
@@ -75,14 +74,12 @@ class Image(object):
             for x in range(size[0] - 1):
                 pix = new_im.getpixel((x, y * 2))
                 pixl = new_im.getpixel((x, y * 2 + 1))
-                print(
-                    colors.bg.rgb(*pixl) & colors.fg.rgb(*pix),
-                    u"\u2580",
-                    sep="",
-                    end="",
+                sys.stdout.write(
+                    (colors.bg.rgb(*pixl) & colors.fg.rgb(*pix)) + "\u2580"
                 )
-            print(colors.reset, " ", sep="")
-        print(colors.reset)
+            sys.stdout.write(colors.reset + " \n")
+        sys.stdout.write(colors.reset + "\n")
+        sys.stdout.flush()
 
 
 class ShowImageApp(cli.Application):
@@ -92,7 +89,7 @@ class ShowImageApp(cli.Application):
     )
 
     @cli.switch(["-c", "--colors"], cli.Range(1, 4), help="Level of color, 1-4")
-    def colors_set(self, n):
+    def colors_set(self, n):  # pylint: disable=no-self-use
         colors.use_color = n
 
     size = cli.SwitchAttr(["-s", "--size"], help="Size, should be in the form 100x150")
@@ -112,4 +109,4 @@ class ShowImageApp(cli.Application):
 
 
 if __name__ == "__main__":
-    ShowImageApp()
+    ShowImageApp.run()
