@@ -1,20 +1,17 @@
-# -*- coding: utf-8 -*-
 """
 Progress bar
 ------------
 """
-from __future__ import division, print_function
 
 import datetime
 import sys
 import warnings
-from abc import abstractmethod
+from abc import ABC, abstractmethod
 
 from plumbum.cli.termsize import get_terminal_size
-from plumbum.lib import six
 
 
-class ProgressBase(six.ABC):
+class ProgressBase(ABC):
     """Base class for progress bars. Customize for types of progress bars.
 
     :param iterator: The iterator to wrap with a progress bar
@@ -72,7 +69,7 @@ class ProgressBase(six.ABC):
         return rval
 
     def next(self):
-        return self.__next__()
+        return next(self)
 
     @property
     def value(self):
@@ -86,7 +83,6 @@ class ProgressBase(six.ABC):
     @abstractmethod
     def display(self):
         """Called to update the progress bar"""
-        pass
 
     def increment(self):
         """Sets next value and displays the bar"""
@@ -110,16 +106,15 @@ class ProgressBase(six.ABC):
         """Returns a string version of time remaining"""
         if self.value < 1:
             return "Starting...                         "
-        else:
-            elapsed_time, time_remaining = list(map(str, self.time_remaining()))
-            return "{} completed, {} remaining".format(
-                elapsed_time.split(".")[0], time_remaining.split(".")[0]
-            )
+
+        elapsed_time, time_remaining = list(map(str, self.time_remaining()))
+        completed = elapsed_time.split(".")[0]
+        remaining = time_remaining.split(".")[0]
+        return f"{completed} completed, {remaining} remaining"
 
     @abstractmethod
     def done(self):
         """Is called when the iterator is done."""
-        pass
 
     @classmethod
     def range(cls, *value, **kargs):
@@ -134,16 +129,17 @@ class ProgressBase(six.ABC):
 
 class Progress(ProgressBase):
     def start(self):
-        super(Progress, self).start()
+        super().start()
         self.display()
 
     def done(self):
         self.value = self.length
         self.display()
         if self.clear and not self.has_output:
-            print("\r", len(str(self)) * " ", "\r", end="", sep="")
+            sys.stdout.write("\r" + len(str(self)) * " " + "\r")
         else:
-            print()
+            sys.stdout.write("\n")
+        sys.stdout.flush()
 
     def __str__(self):
         width = get_terminal_size(default=(0, 0))[0]
@@ -155,21 +151,21 @@ class Progress(ProgressBase):
         ending = " " + (
             self.str_time_remaining()
             if self.timer
-            else "{} of {} complete".format(self.value, self.length)
+            else f"{self.value} of {self.length} complete"
         )
         if width - len(ending) < 10 or self.has_output:
             self.width = 0
+
             if self.timer:
-                return "{:.0%} complete: {}".format(percent, self.str_time_remaining())
-            else:
-                return "{:.0%} complete".format(percent)
+                return f"{percent:.0%} complete: {self.str_time_remaining()}"
 
-        else:
-            self.width = width - len(ending) - 2 - 1
-            nstars = int(percent * self.width)
-            pbar = "[" + "*" * nstars + " " * (self.width - nstars) + "]" + ending
+            return f"{percent:.0%} complete"
 
-        str_percent = " {:.0%} ".format(percent)
+        self.width = width - len(ending) - 2 - 1
+        nstars = int(percent * self.width)
+        pbar = "[" + "*" * nstars + " " * (self.width - nstars) + "]" + ending
+
+        str_percent = f" {percent:.0%} "
 
         return (
             pbar[: self.width // 2 - 2]
@@ -180,11 +176,11 @@ class Progress(ProgressBase):
     def display(self):
         disptxt = str(self)
         if self.width == 0 or self.has_output:
-            print(disptxt)
+            sys.stdout.write(disptxt + "\n")
         else:
-            print("\r", end="")
-            print(disptxt, end="")
-            sys.stdout.flush()
+            sys.stdout.write("\r")
+            sys.stdout.write(disptxt)
+        sys.stdout.flush()
 
 
 class ProgressIPy(ProgressBase):  # pragma: no cover
@@ -200,7 +196,7 @@ class ProgressIPy(ProgressBase):  # pragma: no cover
             except ImportError:  # Support IPython < 4.0
                 from IPython.html.widgets import HTML, HBox, IntProgress
 
-        super(ProgressIPy, self).__init__(*args, **kargs)
+        super().__init__(*args, **kargs)
         self.prog = IntProgress(max=self.length)
         self._label = HTML()
         self._box = HBox((self.prog, self._label))
@@ -209,7 +205,7 @@ class ProgressIPy(ProgressBase):  # pragma: no cover
         from IPython.display import display
 
         display(self._box)
-        super(ProgressIPy, self).start()
+        super().start()
 
     @property
     def value(self):
@@ -220,7 +216,7 @@ class ProgressIPy(ProgressBase):  # pragma: no cover
     def value(self, val):
         self._value = val
         self.prog.value = max(val, 0)
-        self.prog.description = "{:.2%}".format(self.value / self.length)
+        self.prog.description = f"{self.value / self.length:.2%}"
         if self.timer and val > 0:
             self._label.value = self.HTMLBOX.format(self.str_time_remaining())
 
@@ -235,7 +231,7 @@ class ProgressIPy(ProgressBase):  # pragma: no cover
 class ProgressAuto(ProgressBase):
     """Automatically selects the best progress bar (IPython HTML or text). Does not work with qtconsole
     (as that is correctly identified as identical to notebook, since the kernel is the same); it will still
-    iterate, but no graphical indication will be diplayed.
+    iterate, but no graphical indication will be displayed.
 
     :param iterator: The iterator to wrap with a progress bar
     :param length: The length of the iterator (will use ``__len__`` if None)
@@ -246,7 +242,7 @@ class ProgressAuto(ProgressBase):
     def __new__(cls, *args, **kargs):
         """Uses the generator trick that if a cls instance is returned, the __init__ method is not called."""
         try:  # pragma: no cover
-            __IPYTHON__
+            __IPYTHON__  # pylint: disable=pointless-statement
             try:
                 from traitlets import TraitError
             except ImportError:  # Support for IPython < 4.0
@@ -255,7 +251,7 @@ class ProgressAuto(ProgressBase):
             try:
                 return ProgressIPy(*args, **kargs)
             except TraitError:
-                raise NameError()
+                raise NameError() from None
         except (NameError, ImportError):
             return Progress(*args, **kargs)
 
@@ -268,7 +264,7 @@ def main():
     import time
 
     tst = Progress.range(20)
-    for i in tst:
+    for _ in tst:
         time.sleep(1)
 
 
