@@ -1,11 +1,9 @@
-# -*- coding: utf-8 -*-
 """
 This is provided as a quick way to recover your terminal. Simply run
-``python -m plumbum.colorlib``
+``python3 -m plumbum.colorlib``
 to recover terminal color.
 """
 
-from __future__ import absolute_import
 
 from . import main
 
