@@ -1,18 +1,9 @@
-# -*- coding: utf-8 -*-
 import sys
+from io import StringIO
 
 import IPython.display
 from IPython.core.magic import Magics, cell_magic, magics_class, needs_local_scope
 
-if sys.version_info >= (3,):
-    from io import StringIO
-else:
-    try:
-        from cStringIO import StringIO
-    except ImportError:
-        from StringIO import StringIO  # type: ignore
-
-
 valid_choices = [x[8:] for x in dir(IPython.display) if "display_" == x[:8]]
 
 
@@ -27,12 +18,12 @@ class OutputMagics(Magics):  # pragma: no cover
         )
         display_fn = getattr(IPython.display, "display_" + choice)
 
-        "Captures stdout and renders it in the notebook with some ."
+        # Captures stdout and renders it in the notebook
         with StringIO() as out:
             old_out = sys.stdout
             try:
                 sys.stdout = out
-                exec(cell, self.shell.user_ns, local_ns)
+                exec(cell, self.shell.user_ns, local_ns)  # pylint: disable=exec-used
                 out.seek(0)
                 display_fn(out.getvalue(), raw=True)
             finally:
