@@ -1,13 +1,12 @@
-# -*- coding: utf-8 -*-
 """
 Color-related factories. They produce Styles.
-
 """
 
-from __future__ import absolute_import, print_function
 
+import functools
+import operator
 import sys
-from functools import reduce
+from typing import Any
 
 from .names import color_names, default_styles
 from .styles import ColorNotFound
@@ -15,7 +14,7 @@ from .styles import ColorNotFound
 __all__ = ["ColorFactory", "StyleFactory"]
 
 
-class ColorFactory(object):
+class ColorFactory:
     """This creates color names given fg = True/False. It usually will
     be called as part of a StyleFactory."""
 
@@ -35,7 +34,7 @@ class ColorFactory(object):
         try:
             return self._style.from_color(self._style.color_class(item, fg=self._fg))
         except ColorNotFound:
-            raise AttributeError(item)
+            raise AttributeError(item) from None
 
     def full(self, name):
         """Gets the style for a color, using standard name procedure: either full
@@ -54,8 +53,8 @@ class ColorFactory(object):
         """Return the extended color scheme color for a value."""
         if g is None and b is None:
             return self.hex(r)
-        else:
-            return self._style.from_color(self._style.color_class(r, g, b, fg=self._fg))
+
+        return self._style.from_color(self._style.color_class(r, g, b, fg=self._fg))
 
     def hex(self, hexcode):
         """Return the extended color scheme color for a value."""
@@ -75,9 +74,10 @@ class ColorFactory(object):
             (start, stop, stride) = val.indices(256)
             if stop <= 16:
                 return [self.simple(v) for v in range(start, stop, stride)]
-            else:
-                return [self.full(v) for v in range(start, stop, stride)]
-        elif isinstance(val, tuple):
+
+            return [self.full(v) for v in range(start, stop, stride)]
+
+        if isinstance(val, tuple):
             return self.rgb(*val)
 
         try:
@@ -109,17 +109,16 @@ class ColorFactory(object):
         """This will reset the color on leaving the with statement."""
         return self
 
-    def __exit__(self, type, value, traceback):
+    def __exit__(self, _type: Any, _value: Any, _traceback: Any) -> None:
         """This resets a FG/BG color or all styles,
         due to different definition of RESET for the
         factories."""
 
         self.reset.now()
-        return False
 
     def __repr__(self):
         """Simple representation of the class by name."""
-        return "<{}>".format(self.__class__.__name__)
+        return f"<{self.__class__.__name__}>"
 
 
 class StyleFactory(ColorFactory):
@@ -127,7 +126,7 @@ class StyleFactory(ColorFactory):
     imitates the FG ColorFactory to a large degree."""
 
     def __init__(self, style):
-        super(StyleFactory, self).__init__(True, style)
+        super().__init__(True, style)
 
         self.fg = ColorFactory(True, style)
         self.bg = ColorFactory(False, style)
@@ -184,7 +183,7 @@ class StyleFactory(ColorFactory):
                 prev = self
 
         if styleslist:
-            prev = reduce(lambda a, b: a & b, styleslist)
+            prev = functools.reduce(operator.and_, styleslist)
 
         return prev if isinstance(prev, self._style) else prev.reset
 
@@ -202,6 +201,8 @@ class StyleFactory(ColorFactory):
         """Gets colors from an ansi string, returns those colors"""
         return self._style.from_ansi(colored_string, True)
 
-    def load_stylesheet(self, stylesheet=default_styles):
+    def load_stylesheet(self, stylesheet=None):
+        if stylesheet is None:
+            stylesheet = default_styles
         for item in stylesheet:
             setattr(self, item, self.get_colors_from_string(stylesheet[item]))
