@@ -1,16 +1,34 @@
-# -*- coding: utf-8 -*-
+import contextlib
 import functools
 import shlex
 import subprocess
-import sys
-from contextlib import contextmanager
 from subprocess import PIPE, Popen
 from tempfile import TemporaryFile
 from types import MethodType
+from typing import ClassVar
 
 import plumbum.commands.modifiers
 from plumbum.commands.processes import iter_lines, run_proc
-from plumbum.lib import six
+
+__all__ = (
+    "iter_lines",
+    "run_proc",
+    "shquote",
+    "shquote_list",
+    "RedirectionError",
+    "BaseCommand",
+    "Pipeline",
+    "BaseRedirection",
+    "BoundCommand",
+    "BoundEnvCommand",
+    "ConcreteCommand",
+    "ERROUT",
+    "StdinRedirection",
+    "StdoutRedirection",
+    "StderrRedirection",
+    "AppendingStdoutRedirection",
+    "StdinDataRedirection",
+)
 
 
 class RedirectionError(Exception):
@@ -28,13 +46,8 @@ _funnychars = '"`$\\'
 
 def shquote(text):
     """Quotes the given text with shell escaping (assumes as syntax similar to ``sh``)"""
-    text = six.str(text)
-    if sys.version_info >= (3, 3):
-        return shlex.quote(text)
-    else:
-        import pipes
-
-        return pipes.quote(text)
+    text = str(text)
+    return shlex.quote(text)
 
 
 def shquote_list(seq):
@@ -44,7 +57,7 @@ def shquote_list(seq):
 # ===================================================================================================
 # Commands
 # ===================================================================================================
-class BaseCommand(object):
+class BaseCommand:
     """Base of all command objects"""
 
     __slots__ = ("cwd", "env", "custom_encoding", "__weakref__")
@@ -89,10 +102,11 @@ class BaseCommand(object):
         """Creates a bound-command with the given arguments"""
         if not args:
             return self
+
         if isinstance(self, BoundCommand):
             return BoundCommand(self.cmd, self.args + list(args))
-        else:
-            return BoundCommand(self, args)
+
+        return BoundCommand(self, args)
 
     def __call__(self, *args, **kwargs):
         """A shortcut for `run(args)`, returning only the process' stdout"""
@@ -158,7 +172,7 @@ class BaseCommand(object):
         """Runs a command detached."""
         return self.machine.daemonic_popen(self, cwd, stdout, stderr, append)
 
-    @contextmanager
+    @contextlib.contextmanager
     def bgrun(self, args=(), **kwargs):
         """Runs the given command as a context manager, allowing you to create a
         `pipeline <http://en.wikipedia.org/wiki/Pipeline_(computing)>`_ (not in the UNIX sense)
@@ -195,17 +209,15 @@ class BaseCommand(object):
 
         def runner():
             if was_run[0]:
-                return  # already done
+                return None  # already done
             was_run[0] = True
             try:
                 return run_proc(p, retcode, timeout)
             finally:
                 del p.run  # to break cyclic reference p -> cell -> p
-                for f in [p.stdin, p.stdout, p.stderr]:
-                    try:
+                for f in (p.stdin, p.stdout, p.stderr):
+                    with contextlib.suppress(Exception):
                         f.close()
-                    except Exception:
-                        pass
 
         p.run = runner
         yield p
@@ -300,7 +312,7 @@ class BoundCommand(BaseCommand):
         self.args = list(args)
 
     def __repr__(self):
-        return "BoundCommand({!r}, {!r})".format(self.cmd, self.args)
+        return f"BoundCommand({self.cmd!r}, {self.args!r})"
 
     def _get_encoding(self):
         return self.cmd._get_encoding()
@@ -313,7 +325,7 @@ class BoundCommand(BaseCommand):
         return self.cmd.machine
 
     def popen(self, args=(), **kwargs):
-        if isinstance(args, six.string_types):
+        if isinstance(args, str):
             args = [
                 args,
             ]
@@ -321,7 +333,7 @@ class BoundCommand(BaseCommand):
 
 
 class BoundEnvCommand(BaseCommand):
-    __slots__ = ("cmd", "env", "cwd")
+    __slots__ = ("cmd",)
 
     def __init__(self, cmd, env=None, cwd=None):
         self.cmd = cmd
@@ -329,7 +341,7 @@ class BoundEnvCommand(BaseCommand):
         self.cwd = cwd
 
     def __repr__(self):
-        return "BoundEnvCommand({!r}, {!r})".format(self.cmd, self.env)
+        return f"BoundEnvCommand({self.cmd!r}, {self.env!r})"
 
     def _get_encoding(self):
         return self.cmd._get_encoding()
@@ -347,7 +359,7 @@ class BoundEnvCommand(BaseCommand):
             args,
             cwd=self.cwd if cwd is None else cwd,
             env=dict(self.env, **env),
-            **kwargs
+            **kwargs,
         )
 
 
@@ -359,7 +371,7 @@ class Pipeline(BaseCommand):
         self.dstcmd = dstcmd
 
     def __repr__(self):
-        return "Pipeline({!r}, {!r})".format(self.srccmd, self.dstcmd)
+        return f"Pipeline({self.srccmd!r}, {self.dstcmd!r})"
 
     def _get_encoding(self):
         return self.srccmd._get_encoding() or self.dstcmd._get_encoding()
@@ -427,9 +439,9 @@ class Pipeline(BaseCommand):
 
 class BaseRedirection(BaseCommand):
     __slots__ = ("cmd", "file")
-    SYM = None  # type: str
-    KWARG = None  # type: str
-    MODE = None  # type: str
+    SYM: ClassVar[str]
+    KWARG: ClassVar[str]
+    MODE: ClassVar[str]
 
     def __init__(self, cmd, file):
         self.cmd = cmd
@@ -439,7 +451,7 @@ class BaseRedirection(BaseCommand):
         return self.cmd._get_encoding()
 
     def __repr__(self):
-        return "{}({!r}, {!r})".format(self.__class__.__name__, self.cmd, self.file)
+        return f"{self.__class__.__name__}({self.cmd!r}, {self.file!r})"
 
     def formulate(self, level=0, args=()):
         return self.cmd.formulate(level + 1, args) + [
@@ -456,11 +468,11 @@ class BaseRedirection(BaseCommand):
         from plumbum.machines.remote import RemotePath
 
         if self.KWARG in kwargs and kwargs[self.KWARG] not in (PIPE, None):
-            raise RedirectionError("{} is already redirected".format(self.KWARG))
+            raise RedirectionError(f"{self.KWARG} is already redirected")
         if isinstance(self.file, RemotePath):
             raise TypeError("Cannot redirect to/from remote paths")
-        if isinstance(self.file, six.string_types + (LocalPath,)):
-            f = kwargs[self.KWARG] = open(str(self.file), self.MODE)
+        if isinstance(self.file, (str, LocalPath)):
+            f = kwargs[self.KWARG] = open(str(self.file), self.MODE, encoding="utf-8")
         else:
             kwargs[self.KWARG] = self.file
             f = None
@@ -523,7 +535,7 @@ class StdinDataRedirection(BaseCommand):
 
     def formulate(self, level=0, args=()):
         return [
-            "echo {}".format(shquote(self.data)),
+            f"echo {shquote(self.data)}",
             "|",
             self.cmd.formulate(level + 1, args),
         ]
@@ -533,10 +545,10 @@ class StdinDataRedirection(BaseCommand):
         return self.cmd.machine
 
     def popen(self, args=(), **kwargs):
-        if "stdin" in kwargs and kwargs["stdin"] != PIPE:
+        if kwargs.get("stdin") not in (PIPE, None):
             raise RedirectionError("stdin is already redirected")
         data = self.data
-        if isinstance(data, six.unicode_type) and self._get_encoding() is not None:
+        if isinstance(data, str) and self._get_encoding() is not None:
             data = data.encode(self._get_encoding())
         f = TemporaryFile()
         while data:
@@ -544,15 +556,16 @@ class StdinDataRedirection(BaseCommand):
             f.write(chunk)
             data = data[self.CHUNK_SIZE :]
         f.seek(0)
+        kwargs["stdin"] = f
         # try:
-        return self.cmd.popen(args, stdin=f, **kwargs)
+        return self.cmd.popen(args, **kwargs)
         # finally:
         #    f.close()
 
 
 class ConcreteCommand(BaseCommand):
-    QUOTE_LEVEL = None  # type: int
-    __slots__ = ("executable", "custom_encoding")
+    QUOTE_LEVEL: ClassVar[int]
+    __slots__ = ("executable",)
 
     def __init__(self, executable, encoding):
         self.executable = executable
@@ -564,13 +577,13 @@ class ConcreteCommand(BaseCommand):
         return str(self.executable)
 
     def __repr__(self):
-        return "{}({})".format(type(self).__name__, self.executable)
+        return f"{type(self).__name__}({self.executable})"
 
     def _get_encoding(self):
         return self.custom_encoding
 
     def formulate(self, level=0, args=()):
-        argv = [six.str(self.executable)]
+        argv = [str(self.executable)]
         for a in args:
             if a is None:
                 continue
@@ -581,10 +594,17 @@ class ConcreteCommand(BaseCommand):
                     argv.extend(a.formulate(level + 1))
             elif isinstance(a, (list, tuple)):
                 argv.extend(
-                    shquote(b) if level >= self.QUOTE_LEVEL else six.str(b) for b in a
+                    shquote(b) if level >= self.QUOTE_LEVEL else str(b) for b in a
                 )
             else:
-                argv.append(shquote(a) if level >= self.QUOTE_LEVEL else six.str(a))
+                argv.append(shquote(a) if level >= self.QUOTE_LEVEL else str(a))
         # if self.custom_encoding:
-        #    argv = [a.encode(self.custom_encoding) for a in argv if isinstance(a, six.string_types)]
+        #    argv = [a.encode(self.custom_encoding) for a in argv if isinstance(a, str)]
         return argv
+
+    @property
+    def machine(self):
+        raise NotImplementedError()
+
+    def popen(self, args=(), **kwargs):
+        raise NotImplementedError()
