@@ -1,20 +1,13 @@
-# -*- coding: utf-8 -*-
 import atexit
+import contextlib
 import heapq
-import sys
+import math
 import time
+from queue import Empty as QueueEmpty
+from queue import Queue
 from threading import Thread
 
-from plumbum.lib import IS_WIN32, six
-
-if sys.version_info >= (3,):
-    from io import StringIO
-    from queue import Empty as QueueEmpty
-    from queue import Queue
-else:
-    from cStringIO import StringIO
-    from Queue import Empty as QueueEmpty
-    from Queue import Queue
+from plumbum.lib import IS_WIN32
 
 
 # ===================================================================================================
@@ -26,40 +19,23 @@ def _check_process(proc, retcode, timeout, stdout, stderr):
 
 
 def _iter_lines_posix(proc, decode, linesize, line_timeout=None):
-    try:
-        from selectors import EVENT_READ, DefaultSelector
-    except ImportError:
-        # Pre Python 3.4 implementation
-        from select import select
+    from selectors import EVENT_READ, DefaultSelector
 
-        def selector():
-            while True:
-                rlist, _, _ = select([proc.stdout, proc.stderr], [], [], line_timeout)
-                if not rlist and line_timeout:
-                    raise ProcessLineTimedOut(
-                        "popen line timeout expired",
-                        getattr(proc, "argv", None),
-                        getattr(proc, "machine", None),
-                    )
-                for stream in rlist:
-                    yield (stream is proc.stderr), decode(stream.readline(linesize))
-
-    else:
-        # Python 3.4 implementation
-        def selector():
-            sel = DefaultSelector()
-            sel.register(proc.stdout, EVENT_READ, 0)
-            sel.register(proc.stderr, EVENT_READ, 1)
-            while True:
-                ready = sel.select(line_timeout)
-                if not ready and line_timeout:
-                    raise ProcessLineTimedOut(
-                        "popen line timeout expired",
-                        getattr(proc, "argv", None),
-                        getattr(proc, "machine", None),
-                    )
-                for key, mask in ready:
-                    yield key.data, decode(key.fileobj.readline(linesize))
+    # Python 3.4+ implementation
+    def selector():
+        sel = DefaultSelector()
+        sel.register(proc.stdout, EVENT_READ, 0)
+        sel.register(proc.stderr, EVENT_READ, 1)
+        while True:
+            ready = sel.select(line_timeout)
+            if not ready and line_timeout:
+                raise ProcessLineTimedOut(
+                    "popen line timeout expired",
+                    getattr(proc, "argv", None),
+                    getattr(proc, "machine", None),
+                )
+            for key, _mask in ready:
+                yield key.data, decode(key.fileobj.readline(linesize))
 
     for ret in selector():
         yield ret
@@ -74,7 +50,7 @@ def _iter_lines_posix(proc, decode, linesize, line_timeout=None):
 def _iter_lines_win32(proc, decode, linesize, line_timeout=None):
     class Piper(Thread):
         def __init__(self, fd, pipe):
-            super().__init__(name="PlumbumPiper%sThread" % fd)
+            super().__init__(name=f"PlumbumPiper{fd}Thread")
             self.pipe = pipe
             self.fd = fd
             self.empty = False
@@ -132,22 +108,29 @@ else:
 # ===================================================================================================
 # Exceptions
 # ===================================================================================================
-class ProcessExecutionError(EnvironmentError):
+class ProcessExecutionError(OSError):
     """Represents the failure of a process. When the exit code of a terminated process does not
     match the expected result, this exception is raised by :func:`run_proc
     <plumbum.commands.run_proc>`. It contains the process' return code, stdout, and stderr, as
     well as the command line used to create the process (``argv``)
     """
 
-    def __init__(self, argv, retcode, stdout, stderr, message=None):
+    # pylint: disable-next=super-init-not-called
+    def __init__(self, argv, retcode, stdout, stderr, message=None, *, host=None):
+
+        # we can't use 'super' here since OSError only keeps the first 2 args,
+        # which leads to failuring in loading this object from a pickle.dumps.
+        # pylint: disable-next=non-parent-init-called
         Exception.__init__(self, argv, retcode, stdout, stderr)
+
         self.message = message
+        self.host = host
         self.argv = argv
         self.retcode = retcode
-        if six.PY3 and isinstance(stdout, six.bytes):
-            stdout = six.ascii(stdout)
-        if six.PY3 and isinstance(stderr, six.bytes):
-            stderr = six.ascii(stderr)
+        if isinstance(stdout, bytes):
+            stdout = ascii(stdout)
+        if isinstance(stderr, bytes):
+            stderr = ascii(stderr)
         self.stdout = stdout
         self.stderr = stderr
 
@@ -165,6 +148,8 @@ class ProcessExecutionError(EnvironmentError):
             lines = ["Unexpected exit code: ", str(self.retcode)]
         cmd = "\n              | ".join(cmd.splitlines())
         lines += ["\nCommand line: | ", cmd]
+        if self.host:
+            lines += ["\nHost:         | ", self.host]
         if stdout:
             lines += ["\nStdout:       | ", stdout]
         if stderr:
@@ -197,7 +182,7 @@ class CommandNotFound(AttributeError):
     command was not found in the system's ``PATH``"""
 
     def __init__(self, program, path):
-        Exception.__init__(self, program, path)
+        super().__init__(self, program, path)
         self.program = program
         self.path = path
 
@@ -205,7 +190,7 @@ class CommandNotFound(AttributeError):
 # ===================================================================================================
 # Timeout thread
 # ===================================================================================================
-class MinHeap(object):
+class MinHeap:
     def __init__(self, items=()):
         self._items = list(items)
         heapq.heapify(self._items)
@@ -223,7 +208,7 @@ class MinHeap(object):
         return self._items[0]
 
 
-_timeout_queue = Queue()
+_timeout_queue = Queue()  # type: ignore[var-annotated]
 _shutting_down = False
 
 
@@ -236,26 +221,22 @@ def _timeout_thread_func():
                 timeout = max(0, ttk - time.time())
             else:
                 timeout = None
-            try:
+            with contextlib.suppress(QueueEmpty):
                 proc, time_to_kill = _timeout_queue.get(timeout=timeout)
                 if proc is SystemExit:
                     # terminate
                     return
                 waiting.push((time_to_kill, proc))
-            except QueueEmpty:
-                pass
             now = time.time()
             while waiting:
                 ttk, proc = waiting.peek()
                 if ttk > now:
                     break
                 waiting.pop()
-                try:
+                with contextlib.suppress(OSError):
                     if proc.poll() is None:
                         proc.kill()
                         proc._timed_out = True
-                except EnvironmentError:
-                    pass
     except Exception:
         if _shutting_down:
             # to prevent all sorts of exceptions during interpreter shutdown
@@ -275,10 +256,11 @@ def _register_proc_timeout(proc, timeout):
 
 
 def _shutdown_bg_threads():
-    global _shutting_down
+    global _shutting_down  # pylint: disable=global-statement
     _shutting_down = True
     # Make sure this still exists (don't throw error in atexit!)
-    if _timeout_queue:
+    # TODO: not sure why this would be "falsey", though
+    if _timeout_queue:  # type: ignore[truthy-bool]
         _timeout_queue.put((SystemExit, 0))
         # grace period
         bgthd.join(0.1)
@@ -312,9 +294,9 @@ def run_proc(proc, retcode, timeout=None):
     stdout, stderr = proc.communicate()
     proc._end_time = time.time()
     if not stdout:
-        stdout = six.b("")
+        stdout = b""
     if not stderr:
-        stderr = six.b("")
+        stderr = b""
     if getattr(proc, "custom_encoding", None):
         stdout = stdout.decode(proc.custom_encoding, "ignore")
         stderr = stderr.decode(proc.custom_encoding, "ignore")
@@ -329,6 +311,7 @@ def run_proc(proc, retcode, timeout=None):
 BY_POSITION = object()
 BY_TYPE = object()
 DEFAULT_ITER_LINES_MODE = BY_POSITION
+DEFAULT_BUFFER_SIZE = math.inf
 
 
 def iter_lines(
@@ -337,6 +320,7 @@ def iter_lines(
     timeout=None,
     linesize=-1,
     line_timeout=None,
+    buffer_size=None,
     mode=None,
     _iter_lines=_iter_lines,
 ):
@@ -360,25 +344,41 @@ def iter_lines(
                     Raise an :class:`ProcessLineTimedOut <plumbum.commands.ProcessLineTimedOut>` if the timeout has
                     been reached. ``None`` means no timeout is imposed.
 
+    :param buffer_size: Maximum number of lines to keep in the stdout/stderr buffers, in case of a ProcessExecutionError.
+                    Default is ``None``, which defaults to DEFAULT_BUFFER_SIZE (which is infinite by default).
+                    ``0`` will disable bufferring completely.
+
+    :param mode: Controls what the generator yields. Defaults to DEFAULT_ITER_LINES_MODE (which is BY_POSITION by default)
+                - BY_POSITION (default): yields ``(out, err)`` line tuples, where either item may be ``None``
+                - BY_TYPE: yields ``(fd, line)`` tuples, where ``fd`` is 1 (stdout) or 2 (stderr)
+
     :returns: An iterator of (out, err) line tuples.
     """
     if mode is None:
         mode = DEFAULT_ITER_LINES_MODE
 
+    if buffer_size is None:
+        buffer_size = DEFAULT_BUFFER_SIZE
+    buffer_size: int
+
     assert mode in (BY_POSITION, BY_TYPE)
 
     encoding = getattr(proc, "custom_encoding", None) or "utf-8"
-    decode = lambda s: s.decode(encoding, errors="replace").rstrip()
+    decode = lambda s: s.decode(encoding, errors="replace").rstrip()  # noqa: E731
 
     _register_proc_timeout(proc, timeout)
 
-    buffers = [StringIO(), StringIO()]
+    buffers = [[], []]
     for t, line in _iter_lines(proc, decode, linesize, line_timeout):
 
         # verify that the proc hasn't timed out yet
         proc.verify(timeout=timeout, retcode=None, stdout=None, stderr=None)
 
-        buffers[t].write(line + "\n")
+        buffer = buffers[t]
+        if buffer_size > 0:
+            buffer.append(line)
+            if buffer_size < math.inf:
+                del buffer[:-buffer_size]
 
         if mode is BY_POSITION:
             ret = [None, None]
@@ -388,4 +388,4 @@ def iter_lines(
             yield (t + 1), line  # 1=stdout, 2=stderr
 
     # this will take care of checking return code and timeouts
-    _check_process(proc, retcode, timeout, *(s.getvalue() for s in buffers))
+    _check_process(proc, retcode, timeout, *("\n".join(s) + "\n" for s in buffers))
