@@ -1,4 +1,4 @@
-# -*- coding: utf-8 -*-
+import contextlib
 import logging
 import os
 import platform
@@ -7,40 +7,26 @@ import subprocess
 import sys
 import time
 from contextlib import contextmanager
+from subprocess import PIPE, Popen
 from tempfile import mkdtemp
+from typing import Dict, Tuple
 
 from plumbum.commands import CommandNotFound, ConcreteCommand
 from plumbum.commands.daemons import posix_daemonize, win32_daemonize
 from plumbum.commands.processes import iter_lines
-from plumbum.lib import IS_WIN32, ProcInfo, StaticProperty, six
+from plumbum.lib import IS_WIN32, ProcInfo, StaticProperty
 from plumbum.machines.base import BaseMachine, PopenAddons
 from plumbum.machines.env import BaseEnv
 from plumbum.machines.session import ShellSession
 from plumbum.path.local import LocalPath, LocalWorkdir
 from plumbum.path.remote import RemotePath
 
-if sys.version_info[0] >= 3:
-    # python 3 has the new-and-improved subprocess module
-    from subprocess import PIPE, Popen
-
-    has_new_subprocess = True
-else:
-    # otherwise, see if we have subprocess32
-    try:
-        from subprocess32 import PIPE, Popen
-
-        has_new_subprocess = True
-    except ImportError:
-        from subprocess import PIPE, Popen
-
-        has_new_subprocess = False
-
 
 class PlumbumLocalPopen(PopenAddons):
     iter_lines = iter_lines
 
     def __init__(self, *args, **kwargs):
-        self._proc = Popen(*args, **kwargs)
+        self._proc = Popen(*args, **kwargs)  # pylint: disable=consider-using-with
 
     def __iter__(self):
         return self.iter_lines()
@@ -72,8 +58,7 @@ class LocalEnv(BaseEnv):
 
     def __init__(self):
         # os.environ already takes care of upper'ing on windows
-        self._curr = os.environ.copy()
-        BaseEnv.__init__(self, LocalPath, os.path.pathsep)
+        super().__init__(LocalPath, os.path.pathsep, _curr=os.environ.copy())
         if IS_WIN32 and "HOME" not in self and self.home is not None:
             self["HOME"] = self.home
 
@@ -86,11 +71,11 @@ class LocalEnv(BaseEnv):
 
         :returns: The expanded string"""
         prev = os.environ
-        os.environ = self.getdict()
+        os.environ = self.getdict()  # noqa: B003
         try:
             output = os.path.expanduser(os.path.expandvars(expr))
         finally:
-            os.environ = prev
+            os.environ = prev  # noqa: B003
         return output
 
     def expanduser(self, expr):
@@ -100,11 +85,11 @@ class LocalEnv(BaseEnv):
 
         :returns: The expanded string"""
         prev = os.environ
-        os.environ = self.getdict()
+        os.environ = self.getdict()  # noqa: B003
         try:
             output = os.path.expanduser(expr)
         finally:
-            os.environ = prev
+            os.environ = prev  # noqa: B003
         return output
 
 
@@ -125,14 +110,14 @@ class LocalCommand(ConcreteCommand):
         return local
 
     def popen(self, args=(), cwd=None, env=None, **kwargs):
-        if isinstance(args, six.string_types):
+        if isinstance(args, str):
             args = (args,)
         return self.machine._popen(
             self.executable,
             self.formulate(0, args),
             cwd=self.cwd if cwd is None else cwd,
             env=self.env if env is None else env,
-            **kwargs
+            **kwargs,
         )
 
 
@@ -158,6 +143,7 @@ class LocalMachine(BaseMachine):
 
     custom_encoding = sys.getfilesystemencoding()
     uname = platform.uname()[0]
+    _program_cache: Dict[Tuple[str, str], LocalPath] = {}
 
     def __init__(self):
         self._as_user_stack = []
@@ -199,13 +185,19 @@ class LocalMachine(BaseMachine):
 
         :returns: A :class:`LocalPath <plumbum.machines.local.LocalPath>`
         """
+
+        key = (progname, cls.env.get("PATH", ""))
+
+        with contextlib.suppress(KeyError):
+            return cls._program_cache[key]
+
         alternatives = [progname]
         if "_" in progname:
-            alternatives.append(progname.replace("_", "-"))
-            alternatives.append(progname.replace("_", "."))
+            alternatives += [progname.replace("_", "-"), progname.replace("_", ".")]
         for pn in alternatives:
             path = cls._which(pn)
             if path:
+                cls._program_cache[key] = path
                 return path
         raise CommandNotFound(progname, list(cls.env.path))
 
@@ -216,7 +208,7 @@ class LocalMachine(BaseMachine):
         parts2 = [str(self.cwd)]
         for p in parts:
             if isinstance(p, RemotePath):
-                raise TypeError("Cannot construct LocalPath from {!r}".format(p))
+                raise TypeError(f"Cannot construct LocalPath from {p!r}")
             parts2.append(self.env.expanduser(str(p)))
         return LocalPath(os.path.join(*parts2))
 
@@ -239,15 +231,15 @@ class LocalMachine(BaseMachine):
 
         if isinstance(cmd, LocalPath):
             return LocalCommand(cmd)
-        elif not isinstance(cmd, RemotePath):
+
+        if not isinstance(cmd, RemotePath):
             if "/" in cmd or "\\" in cmd:
                 # assume path
                 return LocalCommand(local.path(cmd))
-            else:
-                # search for command
-                return LocalCommand(self.which(cmd))
-        else:
-            raise TypeError("cmd must not be a RemotePath: {!r}".format(cmd))
+            # search for command
+            return LocalCommand(self.which(cmd))
+
+        raise TypeError(f"cmd must not be a RemotePath: {cmd!r}")
 
     def _popen(
         self,
@@ -259,22 +251,10 @@ class LocalMachine(BaseMachine):
         cwd=None,
         env=None,
         new_session=False,
-        **kwargs
+        **kwargs,
     ):
         if new_session:
-            if has_new_subprocess:
-                kwargs["start_new_session"] = True
-            elif IS_WIN32:
-                kwargs["creationflags"] = (
-                    kwargs.get("creationflags", 0) | subprocess.CREATE_NEW_PROCESS_GROUP
-                )
-            else:
-
-                def preexec_fn(prev_fn=kwargs.get("preexec_fn", lambda: None)):
-                    os.setsid()
-                    prev_fn()
-
-                kwargs["preexec_fn"] = preexec_fn
+            kwargs["start_new_session"] = True
 
         if IS_WIN32 and "startupinfo" not in kwargs and stdin not in (sys.stdin, None):
             subsystem = get_pe_subsystem(str(executable))
@@ -294,15 +274,6 @@ class LocalMachine(BaseMachine):
                     sui.dwFlags |= subprocess.STARTF_USESHOWWINDOW  # @UndefinedVariable
                     sui.wShowWindow = subprocess.SW_HIDE  # @UndefinedVariable
 
-        if not has_new_subprocess and "close_fds" not in kwargs:
-            if IS_WIN32 and (
-                stdin is not None or stdout is not None or stderr is not None
-            ):
-                # we can't close fds if we're on windows and we want to redirect any std handle
-                kwargs["close_fds"] = False
-            else:
-                kwargs["close_fds"] = True
-
         if cwd is None:
             cwd = self.cwd
 
@@ -327,7 +298,7 @@ class LocalMachine(BaseMachine):
             stderr=stderr,
             cwd=str(cwd),
             env=env,
-            **kwargs
+            **kwargs,
         )  # bufsize = 4096
         proc._start_time = time.time()
         proc.custom_encoding = self.custom_encoding
@@ -354,12 +325,12 @@ class LocalMachine(BaseMachine):
         """
         if IS_WIN32:
             return win32_daemonize(command, cwd, stdout, stderr, append)
-        else:
-            return posix_daemonize(command, cwd, stdout, stderr, append)
+
+        return posix_daemonize(command, cwd, stdout, stderr, append)
 
     if IS_WIN32:
 
-        def list_processes(self):
+        def list_processes(self):  # pylint: disable=no-self-use
             """
             Returns information about all running processes (on Windows: using ``tasklist``)
 
@@ -369,12 +340,12 @@ class LocalMachine(BaseMachine):
 
             tasklist = local["tasklist"]
             output = tasklist("/V", "/FO", "CSV")
-            if not six.PY3:
-                # The Py2 csv reader does not support non-ascii values
-                output = output.encode("ascii", "ignore")
             lines = output.splitlines()
             rows = csv.reader(lines)
-            header = next(rows)
+            try:
+                header = next(rows)
+            except StopIteration:
+                raise RuntimeError("tasklist must at least have header") from None
             imgidx = header.index("Image Name")
             pididx = header.index("PID")
             statidx = header.index("Status")
@@ -419,11 +390,11 @@ class LocalMachine(BaseMachine):
     def tempdir(self):
         """A context manager that creates a temporary directory, which is removed when the context
         exits"""
-        dir = self.path(mkdtemp())  # @ReservedAssignment
+        new_dir = self.path(mkdtemp())
         try:
-            yield dir
+            yield new_dir
         finally:
-            dir.delete()
+            new_dir.delete()
 
     @contextmanager
     def as_user(self, username=None):
@@ -444,7 +415,7 @@ class LocalMachine(BaseMachine):
                     [
                         "runas",
                         "/savecred",
-                        "/user:{}".format(username),
+                        f"/user:{username}",
                         '"' + " ".join(str(a) for a in argv) + '"',
                     ],
                     self.which("runas"),
