@@ -1,14 +1,11 @@
-# -*- coding: utf-8 -*-
-from __future__ import absolute_import
-
 import itertools
 import operator
 import os
 import warnings
-from abc import abstractmethod, abstractproperty
+from abc import ABC, abstractmethod
 from functools import reduce
 
-from plumbum.lib import six
+FLAGS = {"f": os.F_OK, "w": os.W_OK, "r": os.R_OK, "x": os.X_OK}
 
 
 class FSUser(int):
@@ -23,7 +20,7 @@ class FSUser(int):
         return self
 
 
-class Path(str, six.ABC):
+class Path(str, ABC):
     """An abstraction over file system paths. This class is abstract, and the two implementations
     are :class:`LocalPath <plumbum.machines.local.LocalPath>` and
     :class:`RemotePath <plumbum.path.remote.RemotePath>`.
@@ -32,14 +29,12 @@ class Path(str, six.ABC):
     CASE_SENSITIVE = True
 
     def __repr__(self):
-        return "<{} {}>".format(self.__class__.__name__, str(self))
+        return f"<{self.__class__.__name__} {self}>"
 
-    def __div__(self, other):
+    def __truediv__(self, other):
         """Joins two paths"""
         return self.join(other)
 
-    __truediv__ = __div__
-
     def __getitem__(self, key):
         if type(key) == str or isinstance(key, Path):
             return self / key
@@ -56,16 +51,16 @@ class Path(str, six.ABC):
     def __eq__(self, other):
         if isinstance(other, Path):
             return self._get_info() == other._get_info()
-        elif isinstance(other, str):
+        if isinstance(other, str):
             if self.CASE_SENSITIVE:
                 return str(self) == other
-            else:
-                return str(self).lower() == other.lower()
-        else:
-            return NotImplemented
+
+            return str(self).lower() == other.lower()
+
+        return NotImplemented
 
     def __ne__(self, other):
-        return not (self == other)
+        return not self == other
 
     def __gt__(self, other):
         return str(self) > str(other)
@@ -80,16 +75,11 @@ class Path(str, six.ABC):
         return str(self) <= str(other)
 
     def __hash__(self):
-        if self.CASE_SENSITIVE:
-            return hash(str(self))
-        else:
-            return hash(str(self).lower())
+        return hash(str(self)) if self.CASE_SENSITIVE else hash(str(self).lower())
 
-    def __nonzero__(self):
+    def __bool__(self):
         return bool(str(self))
 
-    __bool__ = __nonzero__
-
     def __fspath__(self):
         """Added for Python 3.6 support"""
         return str(self)
@@ -110,8 +100,10 @@ class Path(str, six.ABC):
         return self.join("../" * count)
 
     def walk(
-        self, filter=lambda p: True, dir_filter=lambda p: True
-    ):  # @ReservedAssignment
+        self,
+        filter=lambda p: True,  # pylint: disable=redefined-builtin
+        dir_filter=lambda p: True,
+    ):
         """traverse all (recursive) sub-elements under this directory, that match the given filter.
         By default, the filter accepts everything; you can provide a custom filter function that
         takes a path as an argument and returns a boolean
@@ -125,10 +117,10 @@ class Path(str, six.ABC):
             if filter(p):
                 yield p
             if p.is_dir() and dir_filter(p):
-                for p2 in p.walk(filter, dir_filter):
-                    yield p2
+                yield from p.walk(filter, dir_filter)
 
-    @abstractproperty
+    @property
+    @abstractmethod
     def name(self):
         """The basename component of this path"""
 
@@ -138,37 +130,45 @@ class Path(str, six.ABC):
         warnings.warn("Use .name instead", FutureWarning)
         return self.name
 
-    @abstractproperty
+    @property
+    @abstractmethod
     def stem(self):
         """The name without an extension, or the last component of the path"""
 
-    @abstractproperty
+    @property
+    @abstractmethod
     def dirname(self):
         """The dirname component of this path"""
 
-    @abstractproperty
+    @property
+    @abstractmethod
     def root(self):
         """The root of the file tree (`/` on Unix)"""
 
-    @abstractproperty
+    @property
+    @abstractmethod
     def drive(self):
         """The drive letter (on Windows)"""
 
-    @abstractproperty
+    @property
+    @abstractmethod
     def suffix(self):
         """The suffix of this file"""
 
-    @abstractproperty
+    @property
+    @abstractmethod
     def suffixes(self):
         """This is a list of all suffixes"""
 
-    @abstractproperty
+    @property
+    @abstractmethod
     def uid(self):
         """The user that owns this path. The returned value is a :class:`FSUser <plumbum.path.FSUser>`
         object which behaves like an ``int`` (as expected from ``uid``), but it also has a ``.name``
         attribute that holds the string-name of the user"""
 
-    @abstractproperty
+    @property
+    @abstractmethod
     def gid(self):
         """The group that owns this path. The returned value is a :class:`FSUser <plumbum.path.FSUser>`
         object which behaves like an ``int`` (as expected from ``gid``), but it also has a ``.name``
@@ -228,7 +228,6 @@ class Path(str, six.ABC):
     @abstractmethod
     def stat(self):
         """Returns the os.stats for a file"""
-        pass
 
     @abstractmethod
     def with_name(self, name):
@@ -244,10 +243,7 @@ class Path(str, six.ABC):
     def preferred_suffix(self, suffix):
         """Adds a suffix if one does not currently exist (otherwise, no change). Useful
         for loading files with a default suffix"""
-        if len(self.suffixes) > 0:
-            return self
-        else:
-            return self.with_suffix(suffix)
+        return self if len(self.suffixes) > 0 else self.with_suffix(suffix)
 
     @abstractmethod
     def glob(self, pattern):
@@ -293,7 +289,7 @@ class Path(str, six.ABC):
         """
 
     @abstractmethod
-    def open(self, mode="r"):
+    def open(self, mode="r", *, encoding=None):
         """opens this path as a file"""
 
     @abstractmethod
@@ -330,11 +326,13 @@ class Path(str, six.ABC):
         """
 
     @staticmethod
-    def _access_mode_to_flags(
-        mode, flags={"f": os.F_OK, "w": os.W_OK, "r": os.R_OK, "x": os.X_OK}
-    ):
+    def _access_mode_to_flags(mode, flags=None):
+        if flags is None:
+            flags = FLAGS
+
         if isinstance(mode, str):
             mode = reduce(operator.or_, [flags[m] for m in mode.lower()], 0)
+
         return mode
 
     @abstractmethod
@@ -364,7 +362,7 @@ class Path(str, six.ABC):
     def unlink(self):
         """Deletes a symbolic link"""
 
-    def split(self, *dummy_args, **dummy_kargs):
+    def split(self, *_args, **_kargs):
         """Splits the path on directory separators, yielding a list of directories, e.g,
         ``"/var/log/messages"`` will yield ``['var', 'log', 'messages']``.
         """
@@ -377,7 +375,7 @@ class Path(str, six.ABC):
 
     @property
     def parts(self):
-        """Splits the directory into parts, including the base directroy, returns a tuple"""
+        """Splits the directory into parts, including the base directory, returns a tuple"""
         return tuple([self.drive + self.root] + self.split())
 
     def relative_to(self, source):
@@ -404,17 +402,18 @@ class Path(str, six.ABC):
         """Same as ``self.relative_to(other)``"""
         return self.relative_to(other)
 
-    def _glob(self, pattern, fn):
+    @staticmethod
+    def _glob(pattern, fn):
         """Applies a glob string or list/tuple/iterable to the current path, using ``fn``"""
         if isinstance(pattern, str):
             return fn(pattern)
-        else:
-            results = []
-            for single_pattern in pattern:
-                results.extend(fn(single_pattern))
-            return sorted(list(set(results)))
 
-    def resolve(self, strict=False):
+        results = []
+        for single_pattern in pattern:
+            results.extend(fn(single_pattern))
+        return sorted(list(set(results)))
+
+    def resolve(self, strict=False):  # pylint:disable=unused-argument
         """Added to allow pathlib like syntax. Does nothing since
         Plumbum paths are always absolute. Does not (currently) resolve
         symlinks."""
@@ -424,9 +423,8 @@ class Path(str, six.ABC):
     @property
     def parents(self):
         """Pathlib like sequence of ancestors"""
-        join = lambda x, y: self._form(x) / y
         as_list = (
-            reduce(join, self.parts[:i], self.parts[0])
+            reduce(lambda x, y: self._form(x) / y, self.parts[:i], self.parts[0])
             for i in range(len(self.parts) - 1, 0, -1)
         )
         return tuple(as_list)
@@ -437,7 +435,7 @@ class Path(str, six.ABC):
         return self.parents[0]
 
 
-class RelativePath(object):
+class RelativePath:
     """
     Relative paths are the "delta" required to get from one path to another.
     Note that relative path do not point at anything, and thus are not paths.
@@ -463,13 +461,13 @@ class RelativePath(object):
         return self.parts[index]
 
     def __repr__(self):
-        return "RelativePath({!r})".format(self.parts)
+        return f"RelativePath({self.parts!r})"
 
     def __eq__(self, other):
         return str(self) == str(other)
 
     def __ne__(self, other):
-        return not (self == other)
+        return not self == other
 
     def __gt__(self, other):
         return str(self) > str(other)
@@ -486,11 +484,9 @@ class RelativePath(object):
     def __hash__(self):
         return hash(str(self))
 
-    def __nonzero__(self):
+    def __bool__(self):
         return bool(str(self))
 
-    __bool__ = __nonzero__
-
     def up(self, count=1):
         return RelativePath(self.parts[:-count])
 
