@@ -1,10 +1,7 @@
-# -*- coding: utf-8 -*-
 import logging
 import os
 import socket
-import sys
 import time
-from copy import deepcopy
 from multiprocessing import Queue
 from threading import Thread
 
@@ -22,7 +19,6 @@ from plumbum import (
     local,
 )
 from plumbum._testtools import skip_on_windows, skip_without_chown
-from plumbum.lib import six
 from plumbum.machines.session import HostPublicKeyUnknown, IncorrectLogin
 
 try:
@@ -40,6 +36,10 @@ def strassert(one, two):
     assert str(one) == str(two)
 
 
+def assert_is_port(port):
+    assert 0 < int(port) < 2**16
+
+
 # TEST_HOST = "192.168.1.143"
 TEST_HOST = "127.0.0.1"
 if TEST_HOST not in ("::1", "127.0.0.1", "localhost"):
@@ -59,9 +59,6 @@ def test_connection():
     SshMachine(TEST_HOST)
 
 
-@pytest.mark.skip(
-    env.LINUX and env.PY[:2] == (3, 5), reason="Doesn't work on 3.5 on Linux on GHA"
-)
 def test_incorrect_login(sshpass):
     with pytest.raises(IncorrectLogin):
         SshMachine(
@@ -93,7 +90,7 @@ class TestRemotePath:
 
     def test_name(self):
         name = RemotePath(self._connect(), "/some/long/path/to/file.txt").name
-        assert isinstance(name, six.string_types)
+        assert isinstance(name, str)
         assert "file.txt" == str(name)
 
     def test_dirname(self):
@@ -154,7 +151,7 @@ class TestRemotePath:
         with self._connect() as rem:
             with rem.tempdir() as dir:
                 p = dir / "foo.txt"
-                p.write(six.b("hello"))
+                p.write(b"hello")
                 # because we're connected to localhost, we expect UID and GID to be the same
                 assert p.uid == os.getuid()
                 assert p.gid == os.getgid()
@@ -271,7 +268,7 @@ class TestRemotePath:
             assert not tmp.exists()
 
 
-class BaseRemoteMachineTest(object):
+class BaseRemoteMachineTest:
     TUNNEL_PROG_AF_INET = r"""import sys, socket
 s = socket.socket()
 s.bind(("", 0))
@@ -362,6 +359,7 @@ s.close()
                 _, out, _ = sh.run("ls -a")
                 assert ".bashrc" in out or ".bash_profile" in out
 
+    @pytest.mark.xfail(env.PYPY, reason="PyPy sometimes fails here", strict=False)
     def test_env(self):
         with self._connect() as rem:
             with pytest.raises(ProcessExecutionError):
@@ -408,7 +406,7 @@ s.close()
         with self._connect() as rem:
             with rem.tempdir() as dir:
                 assert dir.is_dir()
-                data = six.b("hello world")
+                data = b"hello world"
                 (dir / "foo.txt").write(data)
                 assert (dir / "foo.txt").read() == data
 
@@ -421,16 +419,13 @@ s.close()
     def test_iter_lines_timeout(self):
         with self._connect() as rem:
             try:
-                for i, (out, err) in enumerate(
+                for i, (out, err) in enumerate(  # noqa: B007
                     rem["ping"]["-i", 0.5, "127.0.0.1"].popen().iter_lines(timeout=4)
                 ):
                     print("out:", out)
                     print("err:", err)
-            except NotImplementedError:
-                try:
-                    pytest.skip(str(sys.exc_info()[1]))
-                except AttributeError:
-                    return
+            except NotImplementedError as err:
+                pytest.skip(str(err))
             except ProcessTimedOut:
                 assert i > 3
             else:
@@ -439,7 +434,7 @@ s.close()
     def test_iter_lines_error(self):
         with self._connect() as rem:
             with pytest.raises(ProcessExecutionError) as ex:
-                for i, lines in enumerate(rem["ls"]["--bla"].popen()):
+                for i, _lines in enumerate(rem["ls"]["--bla"].popen()):  # noqa: B007
                     pass
                 assert i == 1
             assert "/bin/ls: " in ex.value.stderr
@@ -453,9 +448,9 @@ s.close()
             rfile.delete()
 
 
-def serve_reverse_tunnel(queue):
+def serve_reverse_tunnel(queue, port):
     s = socket.socket()
-    s.bind(("", 12223))
+    s.bind(("", port))
     s.listen(1)
     s2, _ = s.accept()
     data = s2.recv(100).decode("ascii").strip()
@@ -469,7 +464,8 @@ class TestRemoteMachine(BaseRemoteMachineTest):
     def _connect(self):
         return SshMachine(TEST_HOST)
 
-    def test_tunnel(self):
+    @pytest.mark.parametrize("dynamic_lport", [False, True])
+    def test_tunnel(self, dynamic_lport):
 
         for tunnel_prog in (self.TUNNEL_PROG_AF_INET, self.TUNNEL_PROG_AF_UNIX):
             with self._connect() as rem:
@@ -481,20 +477,41 @@ class TestRemoteMachine(BaseRemoteMachineTest):
                 except ValueError:
                     dhost = None
 
-                with rem.tunnel(12222, port_or_socket, dhost=dhost) as tun:
+                if not dynamic_lport:
+                    lport = 12222
+                else:
+                    lport = 0
+
+                with rem.tunnel(lport, port_or_socket, dhost=dhost) as tun:
+                    if not dynamic_lport:
+                        assert tun.lport == lport
+                    else:
+                        assert_is_port(tun.lport)
+                    assert tun.dport == port_or_socket
+                    assert not tun.reverse
+
                     s = socket.socket()
-                    s.connect(("localhost", 12222))
-                    s.send(six.b("world"))
+                    s.connect(("localhost", tun.lport))
+                    s.send(b"world")
                     data = s.recv(100)
                     s.close()
 
                 print(p.communicate())
                 assert data == b"hello world"
 
-    def test_reverse_tunnel(self):
+    @pytest.mark.parametrize("dynamic_dport", [False, True])
+    def test_reverse_tunnel(self, dynamic_dport):
 
+        lport = 12223 + dynamic_dport
         with self._connect() as rem:
-            get_unbound_socket_remote = """import sys, socket
+            queue = Queue()
+            tunnel_server = Thread(target=serve_reverse_tunnel, args=(queue, lport))
+            tunnel_server.start()
+            message = str(time.time())
+
+            if not dynamic_dport:
+
+                get_unbound_socket_remote = """import sys, socket
 s = socket.socket()
 s.bind(("", 0))
 s.listen(1)
@@ -502,20 +519,28 @@ sys.stdout.write(str(s.getsockname()[1]))
 sys.stdout.flush()
 s.close()
 """
-            p = (rem.python["-u"] << get_unbound_socket_remote).popen()
-            remote_socket = p.stdout.readline().decode("ascii").strip()
-            queue = Queue()
-            tunnel_server = Thread(target=serve_reverse_tunnel, args=(queue,))
-            tunnel_server.start()
-            message = str(time.time())
-            with rem.tunnel(12223, remote_socket, dhost="localhost", reverse=True):
+                p = (rem.python["-u"] << get_unbound_socket_remote).popen()
+                remote_socket = p.stdout.readline().decode("ascii").strip()
+            else:
+                remote_socket = 0
+
+            with rem.tunnel(
+                lport, remote_socket, dhost="localhost", reverse=True
+            ) as tun:
+                assert tun.lport == lport
+                if not dynamic_dport:
+                    assert tun.dport == remote_socket
+                else:
+                    assert_is_port(tun.dport)
+                assert tun.reverse
+
                 remote_send_af_inet = """import socket
 s = socket.socket()
 s.connect(("localhost", {}))
 s.send("{}".encode("ascii"))
 s.close()
 """.format(
-                    remote_socket, message
+                    tun.dport, message
                 )
                 (rem.python["-u"] << remote_send_af_inet).popen()
                 tunnel_server.join(timeout=1)
@@ -608,7 +633,7 @@ class TestParamikoMachine(BaseRemoteMachineTest):
     def test_piping(self):
         with self._connect() as rem:
             try:
-                cmd = rem["ls"] | rem["cat"]
+                rem["ls"] | rem["cat"]
             except NotImplementedError:
                 pass
             else:
@@ -627,33 +652,28 @@ class TestParamikoMachine(BaseRemoteMachineTest):
 
     def test_path_open_remote_write_local_read(self):
         with self._connect() as rem:
-            # TODO: once Python 2.6 support is dropped, the nested
-            # with-statements below can be combined using "with x as a, y as b"
-            with rem.tempdir() as remote_tmpdir:
-                with local.tempdir() as tmpdir:
-                    assert remote_tmpdir.is_dir()
-                    assert tmpdir.is_dir()
-                    data = six.b("hello world")
-                    with (remote_tmpdir / "bar.txt").open("wb") as f:
-                        f.write(data)
-                    rem.download((remote_tmpdir / "bar.txt"), (tmpdir / "bar.txt"))
-                    assert (tmpdir / "bar.txt").open("rb").read() == data
+            with rem.tempdir() as remote_tmpdir, local.tempdir() as tmpdir:
+                assert remote_tmpdir.is_dir()
+                assert tmpdir.is_dir()
+                data = b"hello world"
+                with (remote_tmpdir / "bar.txt").open("wb") as f:
+                    f.write(data)
+                rem.download((remote_tmpdir / "bar.txt"), (tmpdir / "bar.txt"))
+                assert (tmpdir / "bar.txt").open("rb").read() == data
 
             assert not remote_tmpdir.exists()
             assert not tmpdir.exists()
 
     def test_path_open_local_write_remote_read(self):
         with self._connect() as rem:
-            # TODO: cf. note on Python 2.6 support above
-            with rem.tempdir() as remote_tmpdir:
-                with local.tempdir() as tmpdir:
-                    assert remote_tmpdir.is_dir()
-                    assert tmpdir.is_dir()
-                    data = six.b("hello world")
-                    with (tmpdir / "bar.txt").open("wb") as f:
-                        f.write(data)
-                    rem.upload((tmpdir / "bar.txt"), (remote_tmpdir / "bar.txt"))
-                    assert (remote_tmpdir / "bar.txt").open("rb").read() == data
+            with rem.tempdir() as remote_tmpdir, local.tempdir() as tmpdir:
+                assert remote_tmpdir.is_dir()
+                assert tmpdir.is_dir()
+                data = b"hello world"
+                with (tmpdir / "bar.txt").open("wb") as f:
+                    f.write(data)
+                rem.upload((tmpdir / "bar.txt"), (remote_tmpdir / "bar.txt"))
+                assert (remote_tmpdir / "bar.txt").open("rb").read() == data
 
             assert not remote_tmpdir.exists()
             assert not tmpdir.exists()
